#include "CFMainWin.H"
#include "KshTextInput.H"
#include "FStrList.H"
#include "FStrPairList.H"
#include "StrUtil.H"
#include "CFCommand.H"
#include "Screen.H"
#include "helpfile.h"
#include "helpaway.h"
#include <stdio.h>
#include <unistd.h>

#include "CFColors.H"		//BEM

#ifdef clear 
#undef clear
#endif

#define ROOM	"Linux, FreeBSD, Solaris:1"

extern Screen *scr;
extern Logger logger;
extern const char *CURFLOO_VER;
extern FStrPairList *config;
extern string commandfile;
extern string iggyfile;

bool ROOM_ACTIVITY_OFF=true;	//BEM
bool CHAT_PMS_OFF=true;		//BEM

#define ABOUT_WDTH	65
#define ABOUT_HGHT	17
#define HELP_WDTH	60
#define HELP_HGHT	15
#define LOGIN_HGHT	13
#define CHATTERS_WDTH	25
#define THROTTLE_WDTH	40
#define THROTTLE_HGHT	14
#define PROFILE_WDTH	70
#define PROFILE_HGHT	20

#define CFMAIN_TRACELOG		"Trace Log"
#define CFMAIN_LOGIN		"Login"
#define CFMAIN_CHATTERS		"Room List"
#define CFMAIN_MENUBAR		"Menu Bar"
#define CFMAIN_STATUSBAR	"Status Bar"
#define CFMAIN_THROTTLEWIN	"Throttle"
#define CFMAIN_CHATWIN		"Chat Window"
#define CFMAIN_PMLIST		"PM List"
#define CFMAIN_TINPUT		"Input"
#define CFMAIN_ABOUT		"About"
#define CFMAIN_HELP		" Curfloo Help "
#define CFMAIN_AWAYHELP		" Away Shorthand Help "
#define CFMAIN_PROFILE		"Yahoo Profile"

#define CFMAIN_CFG_VAL_ON				"true"
#define CFMAIN_CFG_VAL_OFF				"false"
#define CFMAIN_CFG_USER					"user"
#define CFMAIN_CFG_PASSWD				"passwd"
#define CFMAIN_CFG_ROOM					"room"
#define CFMAIN_CFG_AUTOSIGNON				"autosignon"
#define CFMAIN_CFG_AUTOEXIT				"exitonsignoff"
#define CFMAIN_CFG_CHATLOG				"chatlogfile"
#define CFMAIN_CFG_COLOR				"color"
#define CFMAIN_CFG_TISTYLE				"textinputstyle"
#define CFMAIN_CFG_TISTYLE_VAL_KSH			"ksh/vi"
#define CFMAIN_CFG_TISTYLE_VAL_NORMAL			"normal"
#define CFMAIN_CFG_STRIPNL				"stripnewlines"
#define CFMAIN_CFG_AUTOIGGY				"autoignore"
#define CFMAIN_CFG_AILIST				"airegexp"
#define CFMAIN_CFG_FORCELOWER				"forcelower"
#define CFMAIN_CFG_WRAPTXT				"textunderchatbox"
#define CFMAIN_CFG_INPWAIT				"inputwait"
#define CFMAIN_CFG_INPHWAIT				"inputhandlerwait"
#define CFMAIN_CFG_REFRESHWAIT				"refreshwait"
#define CFMAIN_CFG_CHATSERV				"chatserver"
#define CFMAIN_CFG_CHATPORT				"chatport"
#define CFMAIN_CFG_COOKIESERV				"cookieserver"
#define CFMAIN_CFG_COOKIEPORT				"cookieport"

StrPairList shellenv;

CFMainWin::CFMainWin(int wdth, int hght, const string &user,
			const string &passwd, const string &room):
		Window(wdth,hght),
		_signedon(false),
		_done(false),
		_quitonsignoff(false),
		_intmax(0),
		_getonline(false),
		_getoffline(false),
		_forcedoffline(false),
		_stripnl(true),
		_autoiggy(false),
		_wraptxt(true),
		_forcelower(false) {

	_config=config;
	configure(user,passwd,room);
	(*_login).inFocus();
	_active=_login;
	_rebld=true;
	pthread_mutex_init(&_inplock,NULL);
}

CFMainWin::~CFMainWin() {
	delete(_chatarea);
	delete(_rl);
	delete(_ti);
	delete(_statb);
	delete(_menu);
	delete(_login);
	delete(_tracelog);
	delete(_about);
	delete(_help);
	delete(_awayhelp);
	pthread_mutex_destroy(&_inplock);
}

widstat CFMainWin::input(Char inp) {
	widstat rv;
	if ((*_menu).input(inp)==ws_accepted) {
		return (ws_accepted);
	}
	else if ((*_menu).isFocused()) {
		return (ws_accepted);
	}
	if (_widlist.isVisible(CFMAIN_ABOUT)) {
		_widlist.hide(CFMAIN_ABOUT);
		_rebld=true;
	}
	else if ((inp.isEsc()) && (_widlist.isVisible(CFMAIN_HELP))) {
		_widlist.hide(CFMAIN_HELP);
		_rebld=true;
	}
	else if ((inp.isEsc()) && (_widlist.isVisible(CFMAIN_AWAYHELP))) {
		_widlist.hide(CFMAIN_AWAYHELP);
		_rebld=true;
	}
	else if ((inp.isEsc()) && (_widlist.isVisible(CFMAIN_THROTTLEWIN))) {
		handleMenuItem(CFMENU_ATHROT);
	}
	else if (inp.isF1()) {
		handleMenuItem(CFMENU_HELP);
	}
	else if (inp.isF2()) {
		handleMenuItem(CFMENU_VBWTOG);
	}
	else if (inp.isF3()) {
		handleMenuItem(CFMENU_VCHATLST);
	}
	else if (inp.isF4()) {
		handleMenuItem(CFMENU_VPMLST);
	}
	else if (inp.isF5()) {
		handleMenuItem(CFMENU_OPROFILE);
	}
	else if (inp.isF9()) {
		handleMenuItem(CFMENU_ATHROT);
	}
	else if ((inp.isF10()) && (!(*_menu).isFocused())) {
		(*_menu).inFocus();
		_rebld=true;
	}
	else if (inp.isF11()) {
		scr->snapshot("/tmp/scr",(*Window::_grd));
	}
	else if (inp.isF12()) {
		handleMenuItem(CFMENU_VTRCLOG);
	}
	else if ((!inp.isExtended()) && ((inp.isPgUp()) || (inp.isPgDn()) ||
			(inp.isUpArrow()) || (inp.isDownArrow()) ||
			(inp.isHome()) || (inp.isEnd()))) {
		if (_widlist.isVisible(CFMAIN_TRACELOG)) {
			(*_tracelog).input(inp);
			_rebld=(_rebld)||((*_tracelog).update());
		}
		else if (_widlist.isVisible(CFMAIN_HELP)) {
			(*_help).input(inp);
			_rebld=(_rebld)||((*_help).update());
		}
		else if (_widlist.isVisible(CFMAIN_AWAYHELP)) {
			(*_awayhelp).input(inp);
			_rebld=(_rebld)||((*_awayhelp).update());
		}
		else if (_widlist.isVisible(CFMAIN_PROFILE)) {
			(*_profile).input(inp);
			_rebld=(_rebld)||((*_profile).update());
		}
		else if (_widlist.isVisible(CFMAIN_CHATTERS)) {
			(*_rl).input(inp);
			_rebld=(_rebld)||((*_rl).update());
		}
		else if (_widlist.isVisible(CFMAIN_PMLIST)) {
			(*_pmlistwin).input(inp);
			_rebld=(_rebld)||((*_pmlistwin).update());
		}
		else {
			(*_chatarea).input(inp);
			_rebld=(_rebld)||((*_chatarea).update());
		}
	}
	else if ((_widlist.isVisible(CFMAIN_THROTTLEWIN)) &&
		(((rv=(*_throttle).input(inp))==ws_accepted) ||(rv==ws_completed))) {
	}
	else if ((_widlist.isVisible(CFMAIN_LOGIN)) &&
		(((rv=(*_login).input(inp))==ws_accepted) ||(rv==ws_completed))) {
	}
	else {
		rv = ws_rejected;
		if (_widlist.isVisible(CFMAIN_TINPUT)) {
			pthread_mutex_lock(&_inplock);
			if ((rv=(*_ti).input(inp))==ws_completed) {
				(*_command).handleCommand((*_ti).get().getString());
				(*_ti).reset();
			}
			pthread_mutex_unlock(&_inplock);
		}
		if ((rv != ws_accepted) && (rv != ws_completed)) {
			if (inp.isInterrupt()) {
				_done=true;
				if (_intmax++ == 3)
					return (ws_rejected);
				return (ws_accepted);
			}
			_intmax=0;
		}
	}
	return (ws_accepted);
}

int CFMainWin::run() {
	(*_login).setInfo(_user,_passwd,_room,false);
	_yp.setUserPw(_user,_passwd);
	_yp.setDefaultRoom(_room);
	while (!_done && !(_forcedoffline && _quitonsignoff)) {
		if (!_signedon) {
			bool savedata;
			if ((*_login).getInfo(_user,_passwd,_room,savedata)==0) {
				_yp.setUserPw(_user,_passwd);
    			_yp.setDefaultRoom(_room);
				if (savedata) {
					(*_config).add(CFMAIN_CFG_USER,_user);
					(*_config).add(CFMAIN_CFG_PASSWD,_passwd);
					(*_config).add(CFMAIN_CFG_ROOM,_room);
				}
				_getonline=true;
			}
		}
		if (_getonline) {
			_getonline=false;
			getOnline();
			if (_signedon) {
				_widlist.hide(CFMAIN_LOGIN);
				_widlist.show(CFMAIN_CHATWIN);
				_widlist.show(CFMAIN_CHATTERS);
				_widlist.show(CFMAIN_TINPUT);
				(*_ti).inFocus();
			}
		}
		while ((!_done) && (_signedon)) {
        	YahooMessage
            	msg;
			string
				debugMsg;
			bool
				rcvd=false,
				debug=false;

			if (_getoffline || _forcedoffline) {
				_yp.signOff();
				(*_tracelog).log("logoff successfull");
				(*_login).setStatusMessage("Logoff Successfull.  ");
				_widlist.show(CFMAIN_LOGIN);
				_signedon=false;
				_getoffline=false;
				(*_chatarea).printOfflineMsg();
			//	_widlist.hide(CFMAIN_CHATWIN);
				_widlist.hide(CFMAIN_CHATTERS);
				_widlist.hide(CFMAIN_TINPUT);
				break;	
			}
			if ((rcvd=_yp.getEvent(msg))) {
				if ((!(debug=msg.isError())) && (dispatchEvent(msg)))
					continue;
			}
			else if ((rcvd=_yp.getSpeech(msg))) {
				if ((!(debug=msg.isError())) && (dispatchSpeech(msg)))
				continue;
        	}
        	else {
            	usleep(50000);
        	}
			if (debug) {
				debugMsg="Error Packet:";
				(*_statb).setErr();
			}
			else if ((rcvd) && ((*_chatarea).printMsg(msg))) {
				debugMsg="Unknown Packet:";
				debug=true;
			}
			if (debug) {
				unsigned int count=0;
				(*_tracelog).log(debugMsg);
				(*_tracelog).log(msg.hdr());
				while (count < msg.list().size()) {
					(*_tracelog).log(">>>%s",msg.list()[count++].c_str());
				}
			}
        }
		usleep(50000);
	}
	if (_forcedoffline && _quitonsignoff)
		return 1;
	return 0;
}

void CFMainWin::configure(const string &user, const string &passwd,
			const string &room) {
	Char stile, stile2;
	string tmp;
	char buf[128];
	int port;
	long iwait, ihwait, rwait;

	_user=user;
	_passwd=passwd;
	_room=room;
	stile.setBack(cc_red);
	stile.setFore(cc_white);
	_tracelog=new CFConsole(((Window::_width/5)*4),Window::_height/4,true,true);
	(*_tracelog).style(stile);

	if ((*_config).inlist(CFMAIN_CFG_CHATSERV) &&
		(*_config).inlist(CFMAIN_CFG_CHATPORT)) {
		(*_config).get(CFMAIN_CFG_CHATPORT,tmp);
		port=strtol(tmp.c_str(),NULL,10);
		(*_config).get(CFMAIN_CFG_CHATSERV,tmp);
		_yp.setChatServer(tmp,port);
	}
	if ((*_config).inlist(CFMAIN_CFG_COOKIESERV) &&
		(*_config).inlist(CFMAIN_CFG_COOKIEPORT)) {
		(*_config).get(CFMAIN_CFG_COOKIEPORT,tmp);
		port=strtol(tmp.c_str(),NULL,10);
		(*_config).get(CFMAIN_CFG_COOKIESERV,tmp);
		_yp.setCookieServer(tmp,port);
	}
	
	if ((!_user.length()) && ((*_config).inlist(CFMAIN_CFG_USER)))
		(*_config).get(CFMAIN_CFG_USER,_user);
	if ((!_passwd.length()) && ((*_config).inlist(CFMAIN_CFG_PASSWD)))
		(*_config).get(CFMAIN_CFG_PASSWD,_passwd);
	if (!_room.length()) {
		if ((*_config).inlist(CFMAIN_CFG_ROOM))
			(*_config).get(CFMAIN_CFG_ROOM,_room);
		else 
			_room=ROOM;
	}
	if ((*_config).inlist(CFMAIN_CFG_AUTOSIGNON)) {
		(*_config).get(CFMAIN_CFG_AUTOSIGNON,tmp);
		if (!strncasecmp(tmp.c_str(),CFMAIN_CFG_VAL_OFF,
				strlen(CFMAIN_CFG_VAL_OFF))) {
			_getonline=false;
		}
		else if (!strncasecmp(tmp.c_str(),CFMAIN_CFG_VAL_ON,
				strlen(CFMAIN_CFG_VAL_ON))) {
			_getonline=true;
		}
	}
	if ((*_config).inlist(CFMAIN_CFG_AUTOEXIT)) {
		(*_config).get(CFMAIN_CFG_AUTOEXIT,tmp);
		if (!strncasecmp(tmp.c_str(),CFMAIN_CFG_VAL_OFF,
				strlen(CFMAIN_CFG_VAL_OFF))) {
			_quitonsignoff=false;
		}
		else if (!strncasecmp(tmp.c_str(),CFMAIN_CFG_VAL_ON,
				strlen(CFMAIN_CFG_VAL_ON))) {
			_quitonsignoff=true;
		}
	}
	if ((*_config).inlist(CFMAIN_CFG_WRAPTXT)) {
		(*_config).get(CFMAIN_CFG_WRAPTXT,tmp);
		if (!strncasecmp(tmp.c_str(),CFMAIN_CFG_VAL_OFF,
				strlen(CFMAIN_CFG_VAL_OFF))) {
			_wraptxt=false;
		}
		if (!strncasecmp(tmp.c_str(),CFMAIN_CFG_VAL_ON,
				strlen(CFMAIN_CFG_VAL_ON))) {
			_wraptxt=true;
		}
	}
	if (_wraptxt) {
		_chatarea=new CFChatArea(Window::_width-CHATTERS_WDTH,
				Window::_height-4);
	}
	else {
		_chatarea=new CFChatArea(Window::_width,Window::_height-4);
	}
	if ((*_config).inlist(CFMAIN_CFG_CHATLOG)) {
		(*_config).get(CFMAIN_CFG_CHATLOG,tmp);
		if ((*_chatarea).enableLog(tmp))
			(*_tracelog).log("cannot open logfile, logging disabled");
	}

	int count=0;
	sprintf(buf,"%s%02d",CFMAIN_CFG_AILIST,count);
	while ((count < 100) && ((*_config).inlist(buf))) {
		(*_config).get(buf,tmp);
		_ailist.add(tmp);
		sprintf(buf,"%s%02d",CFMAIN_CFG_AILIST,++count);
	}

	if ((*_config).inlist(CFMAIN_CFG_AUTOIGGY)) {
		(*_config).get(CFMAIN_CFG_AUTOIGGY,tmp);
		if (!strncasecmp(tmp.c_str(),CFMAIN_CFG_VAL_OFF,
				strlen(CFMAIN_CFG_VAL_OFF))) {
			_autoiggy=false;
		}
		else if (_ailist.size()) {
			_autoiggy=true;
		}
	}
	else if (_ailist.size()) {
		_autoiggy=true;
	}

	if ((*_config).inlist(CFMAIN_CFG_COLOR)) {
		(*_config).get(CFMAIN_CFG_COLOR,tmp);
		if (!strncasecmp(tmp.c_str(),CFMAIN_CFG_VAL_OFF,
				strlen(CFMAIN_CFG_VAL_OFF))) {
			(*scr).colorOff();
		}
		else if (!strncasecmp(tmp.c_str(),CFMAIN_CFG_VAL_ON,
				strlen(CFMAIN_CFG_VAL_ON))) {
			(*scr).colorOn();
		}
	}
	if ((*_config).inlist(CFMAIN_CFG_TISTYLE)) {
		(*_config).get(CFMAIN_CFG_TISTYLE,tmp);
		if (!strncasecmp(tmp.c_str(),CFMAIN_CFG_TISTYLE_VAL_KSH,
				strlen(CFMAIN_CFG_TISTYLE_VAL_KSH))) {
			_ti=new KshTextInput(Window::_width,512,_peeps,shellenv);
			_inptype=it_kshvi;
		}
		else {
			_ti=new TextInput(Window::_width,512,_peeps);
			_inptype=it_normal;
		}
	}
	else {
		_ti=new TextInput(Window::_width,512,_peeps);
		_inptype=it_normal;
	}
	if ((*_config).inlist(CFMAIN_CFG_STRIPNL)) {
		(*_config).get(CFMAIN_CFG_STRIPNL,tmp);
		if (!strncasecmp(tmp.c_str(),CFMAIN_CFG_VAL_OFF,
				strlen(CFMAIN_CFG_VAL_OFF))) {
			_stripnl=false;
		}
		if (!strncasecmp(tmp.c_str(),CFMAIN_CFG_VAL_ON,
				strlen(CFMAIN_CFG_VAL_ON))) {
			_stripnl=true;
		}
	}
	if ((*_config).inlist(CFMAIN_CFG_FORCELOWER)) {
		(*_config).get(CFMAIN_CFG_FORCELOWER,tmp);
		if (!strncasecmp(tmp.c_str(),CFMAIN_CFG_VAL_OFF,
				strlen(CFMAIN_CFG_VAL_OFF))) {
			_forcelower=false;
		}
		if (!strncasecmp(tmp.c_str(),CFMAIN_CFG_VAL_ON,
				strlen(CFMAIN_CFG_VAL_ON))) {
			_forcelower=true;
		}
	}

	iwait=-1;
	if ((*_config).inlist(CFMAIN_CFG_INPWAIT)) {
		(*_config).get(CFMAIN_CFG_INPWAIT,tmp);
		iwait=strtol(tmp.c_str(),NULL,10);
	}
	ihwait=-1;
	if ((*_config).inlist(CFMAIN_CFG_INPHWAIT)) {
		(*_config).get(CFMAIN_CFG_INPHWAIT,tmp);
		ihwait=strtol(tmp.c_str(),NULL,10);
	}
	rwait=-1;
	if ((*_config).inlist(CFMAIN_CFG_REFRESHWAIT)) {
		(*_config).get(CFMAIN_CFG_REFRESHWAIT,tmp);
		rwait=strtol(tmp.c_str(),NULL,10);
	}
	_throttle=new CFThrottle(THROTTLE_WDTH,THROTTLE_HGHT);
	(*_throttle).setValues(rwait,iwait,ihwait);

	_command=new CFCommand(_yp,*this,commandfile);

	_login=new CFLogin(((Window::_width/3)*2),LOGIN_HGHT,true,true);
	_about=new CFAbout(ABOUT_WDTH,ABOUT_HGHT,true,false);
	_help=new TextWin(HELP_WDTH,HELP_HGHT,true,true,HELP_CONTENTS, CFMAIN_HELP);
	_awayhelp=new TextWin(HELP_WDTH,HELP_HGHT,true,true,HELP_AWAY_CONTENTS,
			CFMAIN_AWAYHELP);
	_pmlistwin=new CFPMListWin(Window::_width,Window::_height/3,256);
	_rl=new CFChattersWin(CHATTERS_WDTH,Window::_height-3,true);
	_statb=new CFStatusBar(Window::_width);
	_profile=new CFProfile(user,PROFILE_WDTH,PROFILE_HGHT,true,false);
	_menu=new CFMenu(Window::_width,this);
	_tiggy=new StrList();
	_piggy=new FStrList(iggyfile);

//	stile.setFore(cc_undef);
//	stile.setBack(cc_undef);
	stile.setFore(CHATAREA_FG_COLOR); //BEM
	stile.setBack(CHATAREA_BG_COLOR); //BEM
	(*_chatarea).style(stile);
	stile.setFore(PMLISTWIN_FG_COLOR); //BEM
	stile.setBack(PMLISTWIN_BG_COLOR); //BEM
	(*_pmlistwin).style(stile);
	stile.setFore(PROFILE_FG_COLOR); //BEM
	stile.setBack(PROFILE_BG_COLOR); //BEM
	(*_profile).style(stile);
	stile.setFore(cc_black);
	stile.setBack(cc_yellow);
	stile2.setFore(cc_yellow);
	stile2.setBack(cc_red);
	(*_throttle).style(stile,stile2);	
	stile.setFore(cc_white);
	stile.setBack(cc_blue);
	(*_rl).style(stile);
	(*_login).style(stile);
	stile.setBack(cc_magenta);
	stile.setFore(cc_yellow);
	stile2=stile;
	stile2.setFore(cc_white);
	(*_help).style(stile,stile2);
	(*_awayhelp).style(stile,stile2);
	stile.setBack(STATUSBAR_BG_COLOR); //BEM
	stile.setFore(STATUSBAR_FG_COLOR); //BEM
	stile.tStandout();
	(*_statb).style(stile);
	stile.setBack(MENUBAR_BG_COLOR); //BEM
	stile.setFore(MENUBAR_FG_COLOR); //BEM
	(*_menu).style(stile);
	add(CFMAIN_MENUBAR,_menu,0,0);
	add(CFMAIN_STATUSBAR,_statb,0,Window::_height-1);
	add(CFMAIN_ABOUT,_about,((Window::_width-ABOUT_WDTH)/2),
			(Window::_height-ABOUT_HGHT)/2);
	add(CFMAIN_TRACELOG,_tracelog,(Window::_width/10),2);
	add(CFMAIN_HELP,_help,((Window::_width-HELP_WDTH)/2),
			(Window::_height-HELP_HGHT)/2);
	add(CFMAIN_AWAYHELP,_awayhelp,((Window::_width-HELP_WDTH)/2),
			(Window::_height-HELP_HGHT)/2);
	add(CFMAIN_THROTTLEWIN,_throttle,((Window::_width-THROTTLE_WDTH)/2),
			(Window::_height-THROTTLE_HGHT)/2);
	add(CFMAIN_PROFILE,_profile,((Window::_width-PROFILE_WDTH)/2),1);
	add(CFMAIN_LOGIN,_login,((Window::_width/3)/2),
			(Window::_height-LOGIN_HGHT)/2);
	add(CFMAIN_TINPUT,_ti,0,Window::_height-2);
	add(CFMAIN_CHATTERS,_rl,(Window::_width-CHATTERS_WDTH),1);
	add(CFMAIN_PMLIST,_pmlistwin,0,1);
	add(CFMAIN_CHATWIN,_chatarea,0,1);
	_widlist.hide(CFMAIN_TINPUT);
	_widlist.hide(CFMAIN_CHATWIN);
	_widlist.hide(CFMAIN_PMLIST);
	_widlist.hide(CFMAIN_CHATTERS);
	_widlist.hide(CFMAIN_THROTTLEWIN);
	_widlist.hide(CFMAIN_PROFILE);
	_widlist.hide(CFMAIN_TRACELOG);
	_widlist.hide(CFMAIN_ABOUT);
	_widlist.hide(CFMAIN_HELP);
	_widlist.hide(CFMAIN_AWAYHELP);
}

int CFMainWin::handleMenuItem(const string &name,StrList params) {
	if (name==CFMENU_QUIT)
		_done=true;	
	else if (name==CFMENU_SIGNON) {
		_getonline=true;
	}
	else if (name==CFMENU_SIGNOFF) {
		_getoffline=true;
	}
	else if (name==CFMENU_LOG) {
	}
	else if (name==CFMENU_ABOUT) {
		_widlist.show(CFMAIN_ABOUT);
		_rebld=true;
	}
	else if (name==CFMENU_HELP) {
		if (_widlist.isVisible(CFMAIN_HELP)) {
			_widlist.hide(CFMAIN_HELP);
		}
		else {
			(*_help).resetPosition();
			_widlist.show(CFMAIN_HELP);
		}
		_rebld=true;
	}
	else if (name==CFMENU_AWAYHELP) {
		(*_awayhelp).resetPosition();
		_widlist.show(CFMAIN_AWAYHELP);
		_rebld=true;
	}
	else if (name==CFMENU_ATHROT) {
		if (_widlist.isVisible(CFMAIN_THROTTLEWIN)) {
			long rwait,iwait,ihwait;
			char buf[32];
			_widlist.hide(CFMAIN_THROTTLEWIN);
			(*_throttle).getValues(rwait,iwait,ihwait);
			sprintf(buf,"%ld",iwait);
			(*_config).add(CFMAIN_CFG_INPWAIT,buf);
			sprintf(buf,"%ld",ihwait);
			(*_config).add(CFMAIN_CFG_INPHWAIT,buf);
			sprintf(buf,"%ld",rwait);
			(*_config).add(CFMAIN_CFG_REFRESHWAIT,buf);
		}
		else {
			_widlist.show(CFMAIN_THROTTLEWIN);
		}
		_rebld=true;
	}
	else if (name==CFMENU_VCHATLST) {
		if (_widlist.isVisible(CFMAIN_CHATTERS)) {
			_widlist.hide(CFMAIN_CHATTERS);
			(*_chatarea).resize(Window::_width, Window::_height-4);
		}
		else {
			_widlist.show(CFMAIN_CHATTERS);
			if (_wraptxt) {
				(*_chatarea).resize(Window::_width-CHATTERS_WDTH,
					Window::_height-4);
			}
		}
		_rebld=true;
	}
	else if (name==CFMENU_VPMLST) {
		if (_widlist.isVisible(CFMAIN_PMLIST))
			_widlist.hide(CFMAIN_PMLIST);
		else {
			_widlist.show(CFMAIN_PMLIST);
			(*_statb).unsetPM();
		}
		_rebld=true;
	}
	else if (name==CFMENU_VTRCLOG) {
		if (_widlist.isVisible(CFMAIN_TRACELOG))
			_widlist.hide(CFMAIN_TRACELOG);
		else {
			_widlist.show(CFMAIN_TRACELOG);
			(*_statb).unsetErr();
		}
		_rebld=true;
	}
	else if (name==CFMENU_CLRSCR) {
		(*_chatarea).clear();
	}
	else if (name==CFMENU_VBWTOG) {
		if (scr->isColor()) {
			scr->colorOff();
			(*_config).add(CFMAIN_CFG_COLOR,CFMAIN_CFG_VAL_OFF);
		}
		else {
			scr->colorOn();
			(*_config).add(CFMAIN_CFG_COLOR,CFMAIN_CFG_VAL_ON);
		}
	}
	else if (name==CFMENU_MSTRIPNL) {
		_stripnl=!_stripnl;
		if (_stripnl) {
			(*_config).add(CFMAIN_CFG_STRIPNL,CFMAIN_CFG_VAL_ON);
		}
		else {
			(*_config).add(CFMAIN_CFG_STRIPNL,CFMAIN_CFG_VAL_OFF);
		}
		(*_chatarea).stripNewlines(_stripnl);	
	}
	else if (name==CFMENU_MFLCASE) {
		_forcelower=!_forcelower;
		if (_forcelower) {
			(*_config).add(CFMAIN_CFG_FORCELOWER,CFMAIN_CFG_VAL_ON);
		}
		else {
			(*_config).add(CFMAIN_CFG_FORCELOWER,CFMAIN_CFG_VAL_OFF);
		}
	}
	else if (name==CFMENU_MWRAPTXT) {
		_wraptxt=!_wraptxt;
		if (_wraptxt) {
			(*_config).add(CFMAIN_CFG_WRAPTXT,CFMAIN_CFG_VAL_ON);
			if (_widlist.isVisible(CFMAIN_CHATTERS)) {
				(*_chatarea).resize(Window::_width-CHATTERS_WDTH,
					Window::_height-4);
			}
		}
		else {
			(*_config).add(CFMAIN_CFG_WRAPTXT,CFMAIN_CFG_VAL_OFF);
			(*_chatarea).resize(Window::_width,Window::_height-4);
		}
	}
	else if (name==CFMENU_MAIGNORE) {
		_autoiggy=!_autoiggy;
		if (_autoiggy) {
			(*_config).add(CFMAIN_CFG_AUTOIGGY,CFMAIN_CFG_VAL_ON);
		}
		else {
			(*_config).add(CFMAIN_CFG_AUTOIGGY,CFMAIN_CFG_VAL_OFF);
		}
	}
	else if (name==CFMENU_TNORMINP) {
		_inptype=it_normal;
		pthread_mutex_lock(&_inplock);
		remove(CFMAIN_TINPUT);
		if (_ti)
			delete (_ti);
		_ti=new TextInput(Window::_width,512,_peeps);
		add(CFMAIN_TINPUT,_ti,0,Window::_height-2);
		pthread_mutex_unlock(&_inplock);
		if (_signedon)
			(*_ti).inFocus();
		else
			_widlist.hide(CFMAIN_TINPUT);
		(*_config).add(CFMAIN_CFG_TISTYLE,CFMAIN_CFG_TISTYLE_VAL_NORMAL);
	}
	else if (name==CFMENU_TKSHINP) {
		_inptype=it_kshvi;
		pthread_mutex_lock(&_inplock);
		remove(CFMAIN_TINPUT);
		if (_ti)
			delete (_ti);
		_ti=new KshTextInput(Window::_width,512,_peeps,shellenv);
		add(CFMAIN_TINPUT,_ti,0,Window::_height-2);
		pthread_mutex_unlock(&_inplock);
		if (_signedon)
			(*_ti).inFocus();
		else
			_widlist.hide(CFMAIN_TINPUT);
		(*_config).add(CFMAIN_CFG_TISTYLE,CFMAIN_CFG_TISTYLE_VAL_KSH);
	}
	else if (name==CFMENU_OBUDDIES) {
		(*_chatarea).printList(_buddies,"Here are your online friends:",
				"(no friends online)");
	}
	else if (name==CFMENU_OCHATTERS) {
		(*_chatarea).printList((*_rl).getList(),"Here's who is in the room:",
				"(you are alone)");
	}
	else if (name==CFMENU_OPMLOG) {
		(*_pmlistwin).printTo(params[0],params[1],true);
	}
	else if (name==CFMENU_OFOLLOW) {
		Text t;    
		Char c;
		c.setFore(cc_cyan);
		t.style(c);
		if (params.size()) {
			char buf[128];
			sprintf(buf,"*** %s will now be followed ***",params[0].c_str());
			t.setText(buf);
			(*_chatarea)+=t;
			if (!_peeps.inlist(params[0])) {
				_yp.gotoUser(params[0]);
			}

		}
		else {
			t.setText("*** follow mode is now off ***");
			(*_chatarea)+=t;
		}
	}
	else if (name==CFMENU_OIGNORE) {
		if (params.size()) {
			Text t;
			Char c;
			char buf[128];

			c.setFore(cc_cyan);
			t.style(c);
			sprintf(buf,"*** %s has been ignored ***",params[0].c_str());
			t.setText(buf);
			(*_chatarea)+=t;
			(*_piggy).add(params[0]);
			StrList sl((*_rl).getList());
			if (sl.inlist(params[0]))
            	(*_rl).add(params[0],true);
		}
		else {
			(*_chatarea).printList((*_piggy),
					"The following chatters have been ignored:",
					"(no chatters ignored)");
		}
	}
	else if (name==CFMENU_OUNIGNORE) {
		Text t;
		Char c;
		char buf[128];

		c.setFore(cc_cyan);
		t.style(c);
		sprintf(buf,"*** %s will no longer be ignored ***",params[0].c_str());
		t.setText(buf);
		(*_chatarea)+=t;
		(*_piggy).remove(params[0]);
		StrList sl((*_rl).getList());
		if (sl.inlist(params[0]))
           	(*_rl).add(params[0],false);
	}
	else if (name==CFMENU_OMUTE) {
		if (params.size()) {
			Text t;
			Char c;
			char buf[128];

			c.setFore(cc_cyan);
			t.style(c);
			sprintf(buf,"*** %s will be temporarily ignored ***",
				params[0].c_str());
			t.setText(buf);
			(*_chatarea)+=t;
			(*_tiggy).add(params[0]);
			StrList sl((*_rl).getList());
			if (sl.inlist(params[0]))
            	(*_rl).add(params[0],true);
		}
		else {
			(*_chatarea).printList((*_tiggy),
					"The following chatters have been temporarily ignored:",
					"(no chatters ignored)");
		}
	}
	else if (name==CFMENU_OUNMUTE) {
		Text t;
		Char c;
		char buf[128];

		c.setFore(cc_cyan);
		t.style(c);
		sprintf(buf,"*** %s will no longer be ignored ***",params[0].c_str());
		t.setText(buf);
		(*_chatarea)+=t;
		(*_tiggy).remove(params[0]);
		StrList sl((*_rl).getList());
		if (sl.inlist(params[0]))
           	(*_rl).add(params[0],false);
	}
	else if (name==CFMENU_OLOCALEXEC) {
		Text t;
		Char c;

		c.setFore(cc_cyan);
		t.style(c);
		t.setText(params[0]);
		(*_chatarea)+=t;
	}
	else if (name==CFMENU_OPROFILE) {
		if (params.size()) {
			(*_profile).fetchProfile(params[0]);
			if (!_widlist.isVisible(CFMAIN_PROFILE))
				_widlist.show(CFMAIN_PROFILE);
		}
		else {
			if (_widlist.isVisible(CFMAIN_PROFILE))
				_widlist.hide(CFMAIN_PROFILE);
			else 
				_widlist.show(CFMAIN_PROFILE);
		}
		_rebld=true;
	}

	// START BEM
	else if (name==CFMENU_OACTIVITY) {
		Text t;
		Char c;

		c.setFore(ROOM_ACTIVITY_TOGGLE_COLOR);
		t.style(c);

		if (ROOM_ACTIVITY_OFF) {
			ROOM_ACTIVITY_OFF=false;
			t.setText(">> Room Activity is now ON");
		} else {
			ROOM_ACTIVITY_OFF=true;
			t.setText(">> Room Activity is now OFF");
		} 
		(*_chatarea)+=t;
	}

	else if (name==CFMENU_OPMS) {
		Text t;
		Char c;

		c.setFore(ROOM_ACTIVITY_TOGGLE_COLOR);
		t.style(c);

		if (CHAT_PMS_OFF) {
			CHAT_PMS_OFF=false;
			t.setText(">> Private Messages are now ON");
		} else {
			CHAT_PMS_OFF=true;
			t.setText(">> Private Messages are now OFF");
		} 
		(*_chatarea)+=t;
	}
	// END BEM

	return 0;
}

bool CFMainWin::isActiveMenuItem(const string &name) {
	if ((name == CFMENU_SIGNON) && (_signedon==true)) {
		return false;
	}
	if ((name == CFMENU_SIGNOFF) && (_signedon==false)) {
		return false;
	}
	if ((name == CFMENU_VCHATLST) && (_signedon==false)) {
		return false;
	}
	if ((name == CFMENU_MAIGNORE) && (!_ailist.size())) {
		return false;
	}
	if ((name == CFMENU_EFRIENDS) || (name == CFMENU_EIGNORE) || 
		(name == CFMENU_EFAVROOM) || (name == CFMENU_ECOLORS) ||
		(name == CFMENU_EBUFSIZE) || (name == CFMENU_EKEYS) ||
		(name == CFMENU_VROOMLST) || (name == CFMENU_EAIGNORE))
		return false;
	return true;
}

string CFMainWin::menuItemText(const string &name) {
	string
		diff;
	if (name==CFMENU_VCHATLST) {
		if (_widlist.isVisible(CFMAIN_CHATTERS))
			diff="+ ";
		else
			diff="- ";
		diff+=name;
	}
	else if (name==CFMENU_VPMLST) {
		if (_widlist.isVisible(CFMAIN_PMLIST))
			diff="+ ";
		else
			diff="- ";
		diff+=name;
	}
	else if (name==CFMENU_VTRCLOG) {
		if (_widlist.isVisible(CFMAIN_TRACELOG))
			diff="+ ";
		else
			diff="- ";
		diff+=name;
	}
	else if (name==CFMENU_VROOMLST) {
		diff="- ";
		diff+=name;
	}
	else if (name==CFMENU_VBWTOG) {
		if (scr->isColor())
			diff="Color Off";
		else
			diff="Color On";
	}
	else if (name==CFMENU_MSTRIPNL) {
		diff=name;
		if (_stripnl)
			diff[0]='*';
	}
	else if (name==CFMENU_MFLCASE) {
		diff=name;
		if (_forcelower)
			diff[0]='*';
	}
	else if (name==CFMENU_MWRAPTXT) {
		diff=name;
		if (!_wraptxt)
			diff[0]='*';
	}
	else if (name==CFMENU_MAIGNORE) {
		diff=name;
		if (_autoiggy)
			diff[0]='*';
	}
	else if (name==CFMENU_TNORMINP) {
// this bug (in Menu.C regarding autosizing of menu's w/ children) 
// NEEDS to be FIXED.   the following is a hack
		diff=name;
		if (_inptype==it_normal)
			diff[0]='*';
//			diff="* ";
//		else
//			diff="  ";
//		diff+=name;
	}
	else if (name==CFMENU_TKSHINP) {
		diff=name;
		if (_inptype==it_kshvi)
			diff[0]='*';

//			diff="* ";
//		else
//			diff="  ";
//		diff+=name;
	}
	else {
		return name;
	}
	return diff;
}

int CFMainWin::getOnline() {
	int counter=0; //BEM
	bool
		gotlogin=false,
		gotbudlist=false;
	YahooMessage
		bl,
		msg;

	(*_login).setStatusMessage("Fetching User Data...");
	if (_yp.fetchCookie()) {
		(*_login).setStatusMessage("Fetching User Data:   Failed.");
		return 1;
	}
	(*_login).setStatusMessage("Logging on to Yahoo...");
	if (_yp.signOnWithMessages()) {
		(*_login).setStatusMessage("Failed to Connect to Yahoo");
		return 2;
	}
	while (!gotlogin) {
		if (_yp.getEvent(msg)) {
			if (msg.isBuddyList()) {
				(*_login).setStatusMessage("Awaiting Acknowledgement..");
				logger.log("got the budlist");
				gotbudlist=true;
				bl=msg;
			} 
			if (msg.isLogin()) {
				gotlogin=true;
				logger.log("got the login");
				(*_login).setStatusMessage("Awaiting Buddy List...");
			}
		}
		else {
			usleep(50000);
		}
	}
	(*_login).setStatusMessage("Joining Room...");
	_yp.joinRoom();
	while (1) {
		if (_yp.getEvent(msg)) {
			if (msg.isRoomList())  {
				(*_login).setStatusMessage("Success");
				_signedon=true;
				break;
			}
			else if (msg.isBuddyList()) {
				logger.log("got the budlist");
				gotbudlist=true;
				bl=msg;
			}
		}
		else {
			// a hack so it doesn't loop forever. BEM
			if (counter > 100) {
				(*_login).setStatusMessage("Problem getting roomlist");
				sleep(1);
				_signedon=true;
				break;
			}
			else {
				counter++;
				usleep(50000);
			}
		}
	}
	if (_signedon) {
		_yp.putEvent(msg);
		if (gotbudlist)
			_yp.putEvent(bl);
		return 0;
	}
	return -1;
}

int CFMainWin::dispatchEvent(const YahooMessage &msg) {
	if (msg.isLogout()) {
		(*_tracelog).log("yahoo has signed you off");
		_forcedoffline=true;
		return 1;
	}
	else if (msg.isConnectionLost()) {
		(*_tracelog).log("the connection to yahoo has been lost");
		_forcedoffline=true;
		return 2;
	}
	else if (msg.isRoomList()) {
		StrList composite(*_piggy);
		composite.add(*_tiggy);
		(*_rl).add(msg.list(),composite);
		_peeps=msg.list();
		_peeps.add(_buddies);
		(*_statb).setRoom(msg.room());
	}
	else if (msg.isRoomEnter()) {
		(*_rl).add(msg.who(),
			((*_piggy).inlist(msg.who()))||((*_tiggy).inlist(msg.who())));
		_peeps.add(msg.who());
	}
	else if (msg.isRoomLeave()) {
		(*_rl).remove(msg.who());
		if (!_buddies.inlist(msg.who()))
			_peeps.remove(msg.who());
		_repost.remove(msg.who());
	}
	else if (msg.isBuddyList()) {
		_buddies=msg.list();
		_peeps.add(msg.list());
	}
	else if (msg.isBuddyOnline()) {
		_buddies.add(msg.who());
		_peeps.add(msg.who());
	}
	else if (msg.isBuddyOffline()) {
		_buddies.remove(msg.who());
		_peeps.remove(msg.who());
	}
	else if (msg.isUserAway()) {
		if (msg.isSelf()) {
			(*_statb).setAway();
		}
		(*_rl).add(msg.who(),
			((*_piggy).inlist(msg.who()))||((*_tiggy).inlist(msg.who())),true);
	}
	else if (msg.isUserBack()) {
		if (msg.isSelf()) {
			(*_statb).unsetAway();
		}
		(*_rl).add(msg.who(),
			((*_piggy).inlist(msg.who()))||((*_tiggy).inlist(msg.who())));
	}
	if (((*_piggy).inlist(msg.who()))||((*_tiggy).inlist(msg.who())))
		return 3;
	return 0;
}

int CFMainWin::dispatchSpeech(YahooMessage &msg) {
	string repeat;
	if ((_repost.get(msg.who(),repeat)==0) && (msg.msg() == repeat))
		return 1;
	if (((*_piggy).inlist(msg.who()))||((*_tiggy).inlist(msg.who()))) {
		if (msg.isPM()) {
			(*_tracelog).log("%s has attempted to PM you",
					msg.who().c_str());
		}
		return 2;
	}
	if (_forcelower) {
		string lwr=msg.msg();
		for (unsigned int x=0; x < lwr.length(); x++)
			lwr[x]=tolower(lwr[x]);
		msg.setmsg(lwr);
	}
	if ((_autoiggy) && (!msg.isSelf()) &&
			(!_buddies.inlist(msg.who()))) {
		StrUtil sutil;
		int rv;
		for (unsigned int j=0; j < _ailist.size(); j++) {
			if ((rv=sutil.grep(_ailist[j],msg.msg()))==0) {
				Char c;
				char tmp[256];
				Text t;

				(*_piggy).add(msg.who());
				sprintf(tmp,"*** %s has been auto-ignored ***",
					msg.who().c_str());
				(*_tracelog).log("%s has been auto-ignored for "
					"saying '%s'", msg.who().c_str(),
					msg.msg().c_str());
				StrList sl((*_rl).getList());
				if (sl.inlist(msg.who()))
					(*_rl).add(msg.who(),true);
				c.setFore(cc_cyan);
				t.style(c);
				t.setText(tmp);
				(*_chatarea)+=t;
				break;
			}
			else if (rv < 0) {
				(*_tracelog).log("Invalid Regexp: %s",
					_ailist[j].c_str());
				_ailist.remove(j);
				break;
			}
		}
		if (((*_piggy).inlist(msg.who()))||((*_tiggy).inlist(msg.who()))) {
			return 3;
		}
	}
	if (msg.isPM()) {
	   if (! CHAT_PMS_OFF) {
		if (!(_widlist.isVisible(CFMAIN_PMLIST)))
			(*_statb).setPM();
		(*_pmlistwin).printTo(msg.who(),msg.msg());
	   }
	}
	_repost.add(msg.who(),msg.msg());
	return 0;
}
